/* serial.c

    Contains parts of code (serial tweaking) which are :
	(C) 2004-5 Captain http://www.captain.at
	http://www.captain.at/electronic-index.php

    Other parts written by Clément NOTIN for Erasme Centre Multimédia
    Summer 2007
*/

#include <termios.h> //serial settings
#include <unistd.h> //write(), read()
#include <stdio.h> //perror()

extern int fd;
extern int BAUDRATE;

//##########################################################
//initialize the serial port with some tweaks
void initport() {
	struct termios options;
	//get the current options for the port
	tcgetattr(fd, &options);
	//set the baud rates to BAUDRATE
	cfsetispeed(&options, BAUDRATE);
	cfsetospeed(&options, BAUDRATE);

    // 8N1
    options.c_cflag &= ~PARENB;
    options.c_cflag &= ~CSTOPB;
    options.c_cflag &= ~CSIZE;
    options.c_cflag |= CS8;
    // no flow control
    options.c_cflag &= ~CRTSCTS;

    options.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
    options.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

    options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
    options.c_oflag &= ~OPOST; // make raw

    // see: http://unixwiz.net/techtips/termios-vmin-vtime.html
    options.c_cc[VMIN]  = 1; //read() will block until there's 1 byte of data available
    //options.c_cc[VTIME] = 0;
    
	//apply the new options for the port
	tcsetattr(fd, TCSANOW, &options);
}



//##########################################################
//write the given one letter code to the serial port
//return 1 if all is good, 0 if writing failed
int writeport(const char *code) {
    int n = write(fd, code, 1);	//arduino will give back the value corresponding to the code
                                //even if *code is a string we don't send more than 1 letter (communication protocol is 1 byte based)
    if (n < 0) {
		perror("write failed ");
		return 0;
	}
	return 1;
}

//##########################################################
//read 1 byte from the serial port in "val"
//return the byte read
int readport() {
	int val=0;
	//maybe the user will send a signal when read() is blocked waiting for datas, there's a bug (buffer isn't freed...) so we manage this here
	//if read() is interrupted it returns -1 but it returns 1 if it successfully read 1 byte so we check this
	while(read(fd,&val,1) != 1);
    return val;
}

